@@ -2,12 +2,12 @@
 '''Proxy Tools'''
 import logging
 
-import trollius
 from trollius import From, Return
+import trollius
 
 from wpull.backport.logging import BraceMessage as __
 from wpull.http.request import Request
-from wpull.recorder import ProgressRecorder
+from wpull.recorder.progress import ProgressRecorder
 
 
 _logger = logging.getLogger(__name__)
@@ -25,9 +25,10 @@ class HTTPProxyServer(object):
         rewrite (bool): If True, strip off URLs ending with
             ``/WPULLHTTPS`` and replaces the scheme with HTTPS.
     '''
-    def __init__(self, http_client, rewrite=False):
+    def __init__(self, http_client, rewrite=False, cookie_jar=None):
         self._http_client = http_client
         self._rewrite = rewrite
+        self._cookie_jar = cookie_jar
 
     @trollius.coroutine
     def __call__(self, reader, writer):
@@ -78,6 +79,9 @@ class HTTPProxyServer(object):
             writer.close()
             return
 
+        if self._cookie_jar:
+            self._cookie_jar.add_cookie_header(request)
+
         if self._rewrite and request.url.endswith('/WPULLHTTPS'):
             request.url = request.url[:-11].replace('http://', 'https://', 1)
 
@@ -89,6 +93,9 @@ class HTTPProxyServer(object):
 
             response = yield From(session.fetch(request))
 
+            if self._cookie_jar:
+                self._cookie_jar.extract_cookies(response, request)
+
             writer.write(response.to_bytes())
             yield From(writer.drain())
             yield From(session.read_content(file=writer, raw=True))
